"""
This module contains all the Naming Authority tools and libraries.
"""
try:
    from ._version import __version__, __version_tuple__
except ImportError:
    try:
        import setuptools_scm
        __version__ = setuptools_scm.get_version()
        del setuptools_scm
    except Exception:
        __version__ = '0.0.0'

__all__ = ['ingest_json', 'update_vocabs', 'util', '__version__', '__url__']

__url__ = 'https://github.com/opengeospatial/ogc-na-tools'



#!/usr/bin/env python3

import unittest
from io import BytesIO
from pathlib import Path

from ogc.na.input_filters import xlsx

THIS_DIR = Path(__file__).parent
DATA_DIR = THIS_DIR / 'data'

with open(DATA_DIR / 'headers.xlsx', 'rb') as f:
    WITH_HEADERS = f.read()
with open(DATA_DIR / 'no-headers.xlsx', 'rb') as f:
    NO_HEADERS = f.read()
with open(DATA_DIR / 'two-sheets.xlsx', 'rb') as f:
    TWO_SHEETS = f.read()


class InputFiltersCSVTest(unittest.TestCase):

    def test_rows_objects(self):
        cfg = {
            'rows': 'dict',
            'skip-rows': 0,
            'header-row': 0,
        }
        rows = xlsx.apply_filter(WITH_HEADERS, cfg)[0]
        self.assertEqual(len(rows), 87)
        self.assertEqual(rows[0], {
            'Year': 1968,
            'Score': 86,
            'Title': 'Greetings',
        })
        self.assertEqual(rows[10], {
            'Year': 1977,
            'Score': 67,
            'Title': 'New York,New York',
        })

        cfg['skip-rows'] = 3
        rows = xlsx.apply_filter(WITH_HEADERS, cfg)[0]
        self.assertEqual(len(rows), 84)
        self.assertEqual(rows[0], {
            'Year': 1971,
            'Score': 40,
            'Title': 'Born to Win',
        })
        self.assertEqual(rows[7], {
            'Year': 1977,
            'Score': 67,
            'Title': 'New York,New York',
        })

        cfg['skip-rows'] = 0
        cfg['header-row'] = 2
        rows = xlsx.apply_filter(WITH_HEADERS, cfg)[0]
        self.assertEqual(len(rows), 85)
        self.assertEqual(rows[0], {
            1970: 1970,
            17: 73,
            'Bloody Mama': 'Hi,Mom!',
        })

    def test_rows_lists(self):
        cfg = {
            'rows': 'list',
            'skip-rows': 0,
        }
        rows = xlsx.apply_filter(WITH_HEADERS, cfg)[0]
        self.assertEqual(len(rows), 88)
        self.assertEqual(rows[0], ['Year', 'Score', 'Title'])
        self.assertEqual(rows[10], [1977, 47, '1900'])
        cfg['skip-rows'] = 1
        rows = xlsx.apply_filter(WITH_HEADERS, cfg)[0]
        self.assertEqual(len(rows), 87)
        self.assertEqual(rows[0], [1968, 86, "Greetings"])
        self.assertEqual(rows[3], [1971, 40, "Born to Win"])
        cfg['header-row'] = 2  # should have no effect
        self.assertEqual(rows, xlsx.apply_filter(WITH_HEADERS, cfg)[0])

        cfg['skip-rows'] = 0
        rows = xlsx.apply_filter(NO_HEADERS, cfg)[0]
        self.assertEqual(len(rows), 6)
        self.assertEqual(rows[0], ['John', 'Doe', '120 jefferson st.', 'Riverside', ' NJ', ' 08075'])

    def test_several_sheets(self):
        rows, metadata = xlsx.apply_filter(TWO_SHEETS, {})
        self.assertEqual(metadata.get('worksheet'), 'FirstSheet')
        self.assertEqual(rows[0], {'FirstCol1': 1, 'FirstCol2': 'DD37Cf93aecA6Dc'})

        rows, metadata = xlsx.apply_filter(TWO_SHEETS, {'worksheet': 'SecondSheet'})
        self.assertEqual(metadata.get('worksheet'), 'SecondSheet')
        self.assertEqual(rows[0], {'SecondCol1': 11, 'SecondCol2': '216E205d6eBb815'})



import json
import unittest
from pathlib import Path

from rdflib import Graph, URIRef, Namespace

from ogc.na import annotate_schema
from ogc.na.annotate_schema import SchemaAnnotator, ContextBuilder, SchemaResolver
from ogc.na.util import load_yaml

THIS_DIR = Path(__file__).parent
DATA_DIR = THIS_DIR / 'data'


def deep_get(dct, *keys):
    for key in keys:
        dct = dct.get(key)
        if dct is None:
            return None
    return dct


class AnnotateSchemaTest(unittest.TestCase):

    def test_resolve_ref_url_full(self):
        ref = 'http://www.example.com/path/to/ref'
        self.assertEqual(annotate_schema.resolve_ref(ref), (None, ref))

    def test_resolve_ref_url_relative(self):
        ref = '/path/to/ref'
        base_url = 'http://www.example.com/base/url'
        self.assertEqual(annotate_schema.resolve_ref(ref, base_url=base_url),
                         (None, 'http://www.example.com/path/to/ref'))

        ref = 'relative/ref'
        self.assertEqual(annotate_schema.resolve_ref(ref, base_url=base_url),
                         (None, 'http://www.example.com/base/relative/ref'))

        ref = '../relative/ref'
        self.assertEqual(annotate_schema.resolve_ref(ref, base_url=base_url),
                         (None, 'http://www.example.com/relative/ref'))

    def test_resolve_ref_filename(self):
        ref = '/tmp/relative/test'
        fn_from = '/var/lib/from.yml'

        self.assertEqual(annotate_schema.resolve_ref(ref, fn_from),
                         (Path(ref), None))

        ref = 'child/ref'
        self.assertEqual(annotate_schema.resolve_ref(ref, fn_from),
                         (Path(fn_from).parent / ref, None))

        ref = '../child/ref2'
        result = annotate_schema.resolve_ref(ref, fn_from)
        self.assertEqual(result[0].resolve(), Path(fn_from).parent.joinpath(ref).resolve(), None)
        self.assertIsNone(result[1])

    def test_annotate_no_follow_refs(self):
        annotator = SchemaAnnotator()
        schema = annotator.process_schema(DATA_DIR / 'sample-schema.yml').schema

        self.assertEqual(deep_get(schema, 'properties', 'propA', 'x-jsonld-id'), 'http://example.com/props/a')
        self.assertEqual(deep_get(schema, 'properties', 'propB', 'x-jsonld-id'), 'http://example.com/props/b')
        self.assertEqual(deep_get(schema, 'properties', 'propC', 'x-jsonld-id'), None)
        self.assertEqual(deep_get(schema, 'properties', 'propD', 'x-jsonld-id'), 'http://example.com/props/d')

    def test_annotate_provided_context(self):
        annotator = SchemaAnnotator()
        schema = annotator.process_schema(DATA_DIR / 'sample-schema.yml', default_context={
            '@context': {
                'another': 'http://example.net/another/',
                'propA': 'another:a',
                'propC': 'another:c'
            }
        }).schema

        self.assertEqual(deep_get(schema, 'properties', 'propA', 'x-jsonld-id'), 'http://example.com/props/a')
        self.assertEqual(deep_get(schema, 'properties', 'propC', 'x-jsonld-id'), 'http://example.net/another/c')

    def test_vocab(self):
        annotator = SchemaAnnotator()
        vocab = 'http://example.com/vocab#'
        schema = annotator.process_schema(DATA_DIR / 'schema-vocab.yml', default_context={
            '@context': {
                '@vocab': vocab,
                'propA': 'test',
                'propB': '@id',
                'propC': 'http://www.another.com/',
            }
        }).schema

        self.assertEqual(deep_get(schema, 'properties', 'propB', 'x-jsonld-id'), '@id')
        self.assertEqual(deep_get(schema, 'properties', 'propC', 'x-jsonld-id'), 'http://www.another.com/')

        builder = ContextBuilder(DATA_DIR / 'schema-vocab.yml', contents=schema)
        instance = {
            **builder.context,
            'propA': 'valueA',
            'propB': 'https://example.com',
            'propC': 'valueC',
            'propD': 'valueD',
        }
        g = Graph().parse(data=json.dumps(instance), format='json-ld')
        resource = URIRef(instance['propB'])
        vocab = Namespace(vocab)
        self.assertEqual(str(g.value(resource, vocab.test)), 'valueA')
        self.assertEqual(str(g.value(resource, URIRef('http://www.another.com/'))), 'valueC')
        self.assertEqual(str(g.value(resource, vocab.propD)), 'valueD')

    def test_top_level_keywords(self):
        annotator = SchemaAnnotator()
        vocab = 'http://example.com/vocab#'
        base = 'http://example.net/'
        schema = annotator.process_schema(DATA_DIR / 'sample-schema-prop-c.yml', default_context={
            '@context': {
                '@base': base,
                '@vocab': vocab,
            }
        }).schema

        self.assertEqual(schema.get('x-jsonld-vocab'), vocab)
        self.assertEqual(schema.get('x-jsonld-base'), base)

        builder = ContextBuilder('http://example.com/schema.yaml', contents=schema)

        self.assertEqual(deep_get(builder.context, '@context', '@vocab'), vocab)
        self.assertEqual(deep_get(builder.context, '@context', '@base'), base)

    def test_schema_anchors(self):
        with open(DATA_DIR / 'schema-anchors.json') as f:
            schema = json.load(f)
        anchors = SchemaResolver._find_anchors(schema)
        self.assertSetEqual({'name', 'age', 'innerProp'}, set(anchors.keys()))

        self.assertEqual(SchemaResolver._get_branch(schema, '#/$defs/name'), anchors.get('name'))
        self.assertEqual(SchemaResolver._get_branch(schema, '#/$defs/age'), anchors.get('age'))
        self.assertEqual(SchemaResolver._get_branch(schema, '#/$defs/deep/properties/inner'),
                         anchors.get('innerProp'))

    def test_defs_annotation(self):
        annotator = SchemaAnnotator()
        orig_schema = load_yaml(DATA_DIR / 'annotate-defs-schema.yml')
        schema = annotator.process_schema(DATA_DIR / 'annotate-defs-schema.yml', contents=orig_schema).schema
        vocab = 'http://example.com/props/'
        self.assertEqual(deep_get(schema, '$defs', 'objectA', 'properties', 'propA', 'x-jsonld-id'),
                         vocab + 'a')
        self.assertEqual(deep_get(schema, '$defs', 'objectB', 'properties', 'propB', 'x-jsonld-id'),
                         vocab + 'b')

        orig_schema = load_yaml(DATA_DIR / 'annotate-defs-schema.yml')
        only_defs_schema = {k: v for k, v in orig_schema.items() if k in ('$schema', '$defs', 'x-jsonld-context')}
        schema = annotator.process_schema(DATA_DIR / 'annotate-defs-schema.yml', contents=only_defs_schema).schema
        self.assertEqual(deep_get(schema, '$defs', 'objectA', 'properties', 'propA', 'x-jsonld-id'),
                         vocab + 'a')
        self.assertEqual(deep_get(schema, '$defs', 'objectB', 'properties', 'propB', 'x-jsonld-id'),
                         vocab + 'b')

    def test_binding_bubbling(self):
        ctx_builder = ContextBuilder(DATA_DIR / 'binding-bubbling/root-schema.yaml')
        self.assertIn('propA1', ctx_builder.context['@context'])
        self.assertIn('propA21', ctx_builder.context['@context'])
        self.assertIn('propB1', ctx_builder.context['@context'])
        self.assertIn('propParent1', ctx_builder.context['@context'])
        self.assertNotIn('propExt1', ctx_builder.context['@context'])
        self.assertNotIn('propExt2', ctx_builder.context['@context'])

        ctx_builder = ContextBuilder(DATA_DIR / 'binding-bubbling/vocab-schema.yaml')
        self.assertIn('propParent1', ctx_builder.context['@context'])
        self.assertIn('propParent2', ctx_builder.context['@context'])
        self.assertIn('propParent21', ctx_builder.context['@context']['propParent2']['@context'])
        self.assertNotIn('propExt1', ctx_builder.context['@context'])
        self.assertNotIn('propExt2', ctx_builder.context['@context'])